/* Includes ------------------------------------------------------------------*/
#include "bsp.h"
#include "timers.h"
#include <math.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Main_Task_PRIO          ( tskIDLE_PRIORITY  + 9 )
#define Main_Task_STACK         ( 3048 )

/* Private macro -------------------------------------------------------------*/
#define A1395_SENSITIVITY       ((double)10.0)

/* Private variables ---------------------------------------------------------*/
xTaskHandle                   Task_Handle;
xTimerHandle                  TouchScreenTimer;

/* Private function prototypes -----------------------------------------------*/
static void Main_Task(void * pvParameters);
static void vTimerCallback(xTimerHandle pxTimer);

/* Private functions ---------------------------------------------------------*/
static void ADC_Config(void){
  ADC_CommonInitTypeDef ADC_CommonInitStructure;
  ADC_InitTypeDef       ADC_InitStructure;
  GPIO_InitTypeDef      GPIO_InitStructure;

  /* Enable ADC interface clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

  /* Enable GPIOC clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);

  /* Configure ADC pin */
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  /* Reset ADC */
  ADC_DeInit();

  /* Configure ADC - common settings */
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
  ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_20Cycles;
  ADC_CommonInit(&ADC_CommonInitStructure);

  /* Configure ADC - settings for ADC1 */
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;  // Select 12-bit resolution
  ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  ADC_InitStructure.ADC_ExternalTrigConv = 0;
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfConversion = 1;              // Select 1 conversion
  ADC_Init(ADC1, &ADC_InitStructure);

  /* Activate ADC */
  ADC_Cmd(ADC1, ENABLE);

  /* Select channel ADC123_IN13 (PC3 pin) */
  ADC_RegularChannelConfig(ADC1, ADC_Channel_13, 1, ADC_SampleTime_144Cycles);
}

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void){
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);

  STM_EVAL_PBInit(BUTTON_USER, BUTTON_MODE_GPIO);

  ADC_Config();

  /* Create main task */
  xTaskCreate(Main_Task,
              (signed char const*)"Magnetometer",
              Main_Task_STACK,
              NULL,
              Main_Task_PRIO,
              &Task_Handle);

  /* Launch Touchscreen Timer */
  TouchScreenTimer = xTimerCreate ((signed char *)"Timer", 50, pdTRUE, ( void * ) 1, vTimerCallback );

  if( TouchScreenTimer != NULL )
  {
    if( xTimerStart( TouchScreenTimer, 0 ) != pdPASS )
    {
      /* The timer could not be set into the Active state. */
    }
  }

  /* Start the FreeRTOS scheduler */
  vTaskStartScheduler();

  return 0;
}

uint16_t adc_convert(){
  // Start conversion of regular channel
  ADC_SoftwareStartConv(ADC1);
  // Wait until end of conversion
  while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
  // Return converted data
  return ADC_GetConversionValue(ADC1);
}

static void _UserDraw(WM_HWIN hWin, int Stage){
  if(Stage == GRAPH_DRAW_LAST) {
    char text[] = "Voltage [V]";
    GUI_RECT Rect, RectInvalid;
    int FontSizeY;
    GUI_SetFont(&GUI_Font13_ASCII);
    FontSizeY = GUI_GetFontSizeY();
    WM_GetInsideRect(&Rect);
    WM_GetInvalidRect(hWin, &RectInvalid);
    Rect.x1 = Rect.x0 + FontSizeY;
    GUI_SetColor(GUI_DARKGREEN);
    GUI_DispStringInRectEx(text, &Rect, GUI_TA_HCENTER, strlen(text), GUI_ROTATE_CCW);
  }
}

/**
  * @brief  Main task - magnetometer application
  * @param  pvParameters not used
  * @retval None
  */
static void Main_Task(void * pvParameters){
  unsigned int ticks = 0;

  // Handles
  FRAMEWIN_Handle    hFrame, hClientWin;
  GRAPH_Handle       hGraph;
  GRAPH_DATA_Handle  hGraphData;
  GRAPH_SCALE_Handle hGraphScale;
  TEXT_Handle        hText_1, hText_2, hText_3, hText_4;

  uint16_t conversion_value;

  double voltage_mV, voltage_V;
  double magneticField_G, magneticField_mT;

  char text[100];

  // Initialize the BSP layer
  LowLevel_Init();

  // Initialize the STemWin GUI Library
  GUI_Init();

  // Set background color
  WM_SetDesktopColor(GUI_LIGHTBLUE);

  // Set FLEX skin for FRAMEWIN widget
  FRAMEWIN_SetDefaultSkin(FRAMEWIN_SKIN_FLEX);

  // Create FRAMEWIN widgets
  hFrame = FRAMEWIN_CreateEx(10, 30, 220, 230, WM_HBKWIN, WM_CF_SHOW, 0, 0, "Magnetometer", NULL);
  hClientWin = WM_GetClientWindow(hFrame);
  FRAMEWIN_SetFont(hFrame, &GUI_Font16_ASCII);
  FRAMEWIN_SetTextAlign(hFrame, GUI_TA_HCENTER);

  // Create GRAPH widget and set it's attributes
  hGraph = GRAPH_CreateEx(5, 5, 200, 120, hClientWin, WM_CF_SHOW, 0, GUI_ID_GRAPH0);
  GRAPH_SetBorder(hGraph, 40, 0, 0, 0);
  GRAPH_SetGridDistY(hGraph, 15);
  GRAPH_SetGridOffY(hGraph, 10);
  GRAPH_SetGridVis(hGraph, 1);
  GRAPH_SetLineStyleH(hGraph, GUI_LS_DOT);
  GRAPH_SetLineStyleV(hGraph, GUI_LS_DOT);
  GRAPH_SetUserDraw(hGraph, _UserDraw);
  // Create data object
  hGraphData = GRAPH_DATA_YT_Create(GUI_GREEN, 200, NULL, 0);
  GRAPH_DATA_YT_SetOffY(hGraphData, 10);
  // Attach data object to GRAPH widget
  GRAPH_AttachData(hGraph, hGraphData);
  // Create scale object
  hGraphScale = GRAPH_SCALE_Create(38, GUI_TA_RIGHT, GRAPH_SCALE_CF_VERTICAL, 30);
  GRAPH_SCALE_SetFactor(hGraphScale, 0.033);
  GRAPH_SCALE_SetNumDecs(hGraphScale, 1);
  GRAPH_SCALE_SetOff(hGraphScale, 10);
  GRAPH_SCALE_SetTextColor(hGraphScale, GUI_BLACK);
  // Attach scale object to GRAPH widget
  GRAPH_AttachScale(hGraph, hGraphScale);

  // Create TEXT widgets
  hText_1 = TEXT_CreateEx(10, 140, 100, 20, hClientWin, WM_CF_SHOW, TEXT_CF_LEFT, GUI_ID_TEXT0, "[V]");
  hText_2 = TEXT_CreateEx(10, 160, 100, 20, hClientWin, WM_CF_SHOW, TEXT_CF_LEFT, GUI_ID_TEXT1, "[G]");
  hText_3 = TEXT_CreateEx(10, 180, 100, 20, hClientWin, WM_CF_SHOW, TEXT_CF_LEFT, GUI_ID_TEXT2, "[mT]");
  hText_4 = TEXT_CreateEx(160, 160, 20, 20, hClientWin, WM_CF_SHOW, TEXT_CF_LEFT, GUI_ID_TEXT3, " ");
  TEXT_SetFont(hText_4, &GUI_Font20_ASCII);

  // Select main window
  WM_SetFocus(hFrame);

  // Run the main task
  while(1){
    if(ticks++ > 5){
      ticks = 0;

      // Toggle LED3 each 50ms
      STM_EVAL_LEDToggle(LED3);

      // Read data
      conversion_value = adc_convert();
      // Convert raw data to [V]
      voltage_mV = (double)conversion_value * 0.7324;
      voltage_V = voltage_mV / 1000.0;
      // Convert to [G] and [mT]
      magneticField_G = (voltage_mV - 1500) / A1395_SENSITIVITY;
      magneticField_mT = magneticField_G / 10.0;

      // Scale and add new data to GRAPH's data object
      GRAPH_DATA_YT_AddValue(hGraphData, 30*voltage_V);

      // Update TEXT objects
      sprintf(text,"V = %.3f [V]", voltage_V);
      TEXT_SetText(hText_1, text);
      sprintf(text,"B = %.3f [G]", magneticField_G);
      TEXT_SetText(hText_2, text);
      sprintf(text,"B = %.3f [mT]", magneticField_mT);
      TEXT_SetText(hText_3, text);

      if(magneticField_G > 5.0){
        TEXT_SetText(hText_4, "N");
        TEXT_SetTextColor(hText_4, GUI_BLUE);
      }
      else if(magneticField_G < -5.0){
        TEXT_SetText(hText_4, "S");
        TEXT_SetTextColor(hText_4, GUI_RED);
      }
      else{
        TEXT_SetText(hText_4, " ");
      }
    }
    GUI_Delay(10);
  }

  WM_DeleteWindow(hGraph);
  WM_DeleteWindow(hFrame);
}

/**
  * @brief  Timer callbacsk
  * @param  pxTimer
  * @retval None
  */
static void vTimerCallback( xTimerHandle pxTimer ){
   BSP_Pointer_Update();
}

/**
  * @brief  Error callback function
  * @param  None
  * @retval None
  */
void vApplicationMallocFailedHook( void ){
  while (1)
  {}
}
